
;; ---> dfinit des macros pour incorporer du calc dans du code ordinaire
;;      (et initialise le systme)

(provide 'calc-stef)
(or 
 (require 'calc nil nil)  ;; emacs 22 or newer
 (load "calc-math"))      ;; hack

(setq calc-internal-prec 8)         ;; prcision
(setq calc-angle-mode 'rad)         ;; angles mesurs en radian

(defvar calc-defmath-depth 0) 

(defmacro with-raw-defmath (&rest body)
  "Evaluate BODY within a \\[defmath] and returns the result"
 `(let (result)
    (incf calc-defmath-depth)
    (unwind-protect
	(calc-wrapper
	 (setq result (apply (defmath dummy () ,@body) ())))
      (decf calc-defmath-depth))
    result))

;;; ???  tester extensivement !

(defmacro with-alg-defmath (&rest body)
  "Evaluate BODY within a \\[defmath] and returns the result as a string (algebraic syntax)"
  (if (zerop calc-defmath-depth)
      `(calc-eval (with-raw-defmath ,@body))
    `(with-raw-defmath ,@body)))

(defmacro with-defmath (&rest body)
  "Evaluates BODY within a `defmath' and returns a number.
Be careful as, if the evaluation returns a Calc data type not readable as a number, the macro will return 0
So a 0 output is either the result or an error signal. This should be improved I guess..."
  (if (zerop calc-defmath-depth)
      `(string-to-number (with-alg-defmath ,@body))
    `(with-raw-defmath ,@body)))

;;;; ????
(put 'with-raw-defmath 'lisp-indent-function 0)
(put 'with-alg-defmath 'lisp-indent-function 0)
(put 'with-defmath 'lisp-indent-function 0)

(defmacro with-raw-defmath-import (vlist &rest body)
  "Similar to \\[with-raw-defmath], but allows in VLIST the importation of symbols which will refer to the same value inside the defmath as outside (sort of \"globals\")."
  `(let ,(mapcar (lambda (v) 
		   (list (intern (concat "var-" (symbol-name v)))
			 (list 'math-read-number (list 'prin1-to-string v))))
		 vlist)
     (with-raw-defmath ,@body)))

(defmacro with-alg-defmath-import (vlist &rest body)
  "Similar to \\[with-alg-defmath], but allows in VLIST the importation of symbols which will refer to the same value inside the defmath as outside (sort of \"globals\")."
  `(let ,(mapcar (lambda (v) 
		   (list (intern (concat "var-" (symbol-name v)))
			 (list 'math-read-number (list 'prin1-to-string v))))
		 vlist)
     (with-alg-defmath ,@body)))

(defmacro with-defmath-import (vlist &rest body)
  "Similar to \\[with-defmath], but allows in VLIST the importation of symbols which will refer to the same value inside the defmath as outside (sort of \"global\")."
  `(let ,(mapcar (lambda (v) 
		   (list (intern (concat "var-" (symbol-name v))) 
			 (list 'math-read-number (list 'prin1-to-string v))))
		 vlist)
     (with-defmath ,@body)))

(put 'with-raw-defmath-import 'lisp-indent-function 1)
(put 'with-alg-defmath-import 'lisp-indent-function 1)
(put 'with-defmath-import 'lisp-indent-function 1)


;une opration  la noix pour lancer le shmilblick :
(calc-eval (apply (defmath dummy () (/ 1 2)) ()))



;; ============================================== Util.:

(defmath max-value (v)        ;; v vector or list, possibly nested 
  (let ((m nil))
    (for (i 1 (length v))
	 (setq tmp (elt v (- i 1)))
	 (if (and (not (realp tmp))
		  (listp tmp))
	     (setq tmp (max-value tmp)))
	 (if (and (realp tmp)
		  (or (null m) (> tmp m)))
	     (setq m tmp)))
    m))


(defmath min-value (v)       
  (let ((m nil))
    (for (i 1 (length v))
	 (setq tmp (elt v (- i 1)))
	 (if (and (not (realp tmp))
		  (listp tmp))
	     (setq tmp (min-value tmp)))
	 (if (and (realp tmp)
		  (or (null m) (< tmp m)))
	     (setq m tmp)))
    m))


;le concat de calc n'accepte que 2 arguments, donc:

(defmath m-concat (&rest args)
  (let ((res ""))
    (while args
      (if (car args)
	  (setq res (concat res (car args))))
      (setq args (cdr args)))
    res))


